/**
 ** Copyright (c) 2002 Sun Microsystems, Inc.
 **
 ** See the file "license.terms" for information on usage and redistribution
 ** of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 **/

/**
 **  gcspy_cont_area.cpp
 **
 **  Utilities for manipulating stream data in a contiguous memory area
 **/

#include "gcspy_cont_area.hpp"

gcspyContArea::gcspyContArea (char         *start,
			      char         *end,
			      gcspy_index_t first_index,
			      gcspy_size_t  block_size) {
  _start = start;
  _end = end;
  _first_index = first_index;
  _block_size = block_size;
  _block_num = calc_block_num(start, end, block_size);
}

void
gcspyContArea::setup_range_strings (gcspySpace *space) {
  char buffer[128];
  for (int i = 0; i < _block_num; ++i) {
    space->set_tile_name(i, generate_range_string(i, buffer));
  }
}

gcspy_length_t
gcspyContArea::calc_block_num (char *start, char *end,
			       gcspy_size_t block_size) {
  gcspy_size_t diff = (gcspy_size_t) end - (gcspy_size_t) start; 
  gcspy_length_t blocks = diff / block_size;
  if ((diff % block_size) != 0)
    ++blocks;
  return blocks;
}

void
gcspyContArea::set_perc (char *start, char *end, int *array) {
  gcspy_size_t curr_size, total_size = 0;
  gcspy_index_t start_index = get_index(start);
  gcspy_index_t end_index   = get_index(end - 1);
  int *curr = &array[start_index];
  for (int i = start_index; i <= end_index; ++i) {
    if ((i == start_index) && (i == end_index)) {
      curr_size = (gcspy_size_t) end - (gcspy_size_t) start;
    } else if (i == start_index) {
      curr_size = (gcspy_size_t) get_addr(i+1) - (gcspy_size_t) start;
    } else if (i == end_index) {
      curr_size = (gcspy_size_t) end  - (gcspy_size_t) get_addr(i);
    } else {
      curr_size = _block_size;
    }

    *curr += curr_size;
    gcspy_assert(*curr <= _block_size);
    total_size += curr_size;

    ++curr;
  }

  gcspy_assert(total_size == ((gcspy_size_t) end - (gcspy_size_t) start));
}

void
gcspyContArea::add_one (char *start, char *end, int *array) {
  gcspy_index_t start_index = get_index(start);
  gcspy_index_t end_index = get_index(end - 1);
  int *curr = &array[start_index];
  for (int i = start_index; i <= end_index; ++i) {
    *curr += 1;
    ++curr;
  }
}

void
gcspyContArea::add_one (char *addr, int *array) {
  gcspy_index_t index = get_index(addr);
  ++array[index];
}

void
gcspyContArea::update_enum_asc (char *addr, int val, char *array) {
  gcspy_index_t index = get_index(addr);
  char *curr = &array[index];
  if (val > *curr)
    *curr = val;
}

void
gcspyContArea::update_enum_desc (char *addr, int val, char *array) {
  gcspy_index_t index = get_index(addr);
  char *curr = &array[index];
  if (val < *curr)
    *curr = val;
}

void
gcspyContArea::resize (char *end) {
  _end = end;
  _block_num = calc_block_num(_start, end, _block_size);
}
