/**
 ** Copyright (c) 2002 Sun Microsystems, Inc.
 **
 ** See the file "license.terms" for information on usage and redistribution
 ** of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 **/

/**
 **  hs_old_driver.cpp
 **
 **  Driver for the HS old generation M&C collector
 **/

#include "hs_perm_driver.hpp"

void
hsPermDriver::retrieve_data_arrays () {
  hsMCDriver::retrieve_data_arrays();
  _alloc_data           = _alloc_stream->get_int_data();
  _refs_to_old_data     = _refs_to_old_stream->get_int_data();
}

void
hsPermDriver::init_summaries (gcspy_size_t area_size) {
  hsMCDriver::init_summaries(area_size);
  _alloc_summary[1] = area_size;
}

hsPermDriver::hsPermDriver (gcspy_size_t block_size,
			    gcspy_size_t card_size,
			    char *start,
			    char *end)
  : hsMCDriver("Permanent Generation", block_size, card_size, start, end) {

  add_used_space_stream();
  add_dead_space_stream();
  add_card_table_stream();

  _alloc_stream = new gcspyStream(gcspyStream::int_type,
				  "Allocation",
				  0, block_size, 0, 0,
				  "Allocation: ", " bytes",
				  gcspyStream::presentation_percent,
				  gcspyStream::paint_style_zero,
				  0, gcspyColorDB::get_color("Magenta"), 1);
  add_stream(_alloc_stream);
  _alloc_summary = _alloc_stream->get_summary();

  add_object_stream();
  add_marking_stream();
  add_refs_stream();
  add_refs_to_young_stream();

  _refs_to_old_stream = new gcspyStream(gcspyStream::int_type,
				     "Reference Fields (Old)",
				     0,
				     gcspyContArea::refs_per_block(block_size),
				     0, 0,
				     "Reference Fields: ", " (Old)",
				     gcspyStream::presentation_plus,
				     gcspyStream::paint_style_zero,
				     0, gcspyColorDB::get_color("Green"), 1);
  add_stream(_refs_to_old_stream);
  _refs_to_old_summary = _refs_to_old_stream->get_summary();

  add_internal_refs_stream();

  retrieve_data_arrays();
}
