/**
 ** Copyright (c) 2002 Sun Microsystems, Inc.
 **
 ** See the file "license.terms" for information on usage and redistribution
 ** of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 **/

/**
 **  test_hs_old_driver.cpp
 **
 **  Test for the HS old generation M&C driver
 **/

#include <stdio.h>
#include <stdlib.h>

#include "gcspy_main_server.hpp"
#include "hs_old_driver.hpp"

#ifdef _SOLARIS_
#include <thread.h>
#endif //_SOLARIS_

#ifdef _LINUX_
#include <pthread.h>
#endif //_LINUX_

#define MAX_LEN        ( 128 * 1024 )
#define BLOCK_SIZE      ( 16 * 1024 )
#define CARD_SIZE             ( 512 )

#define START            ((char *)     ( 1024 * 1024 ))

#define END_ONE          ((char *) ( 2 * 1024 * 1024 ))
#define LIMIT_ONE        ( START + ( 512 * 1024 ) )

#define END_TWO          ((char *) ( 3 * 1024 * 1024 ))
#define LIMIT_TWO        ( START + ( 1024 * 1024 ) )

#define END_THREE        ((char *) ( 4 * 1024 * 1024 ))
#define LIMIT_THREE      ( START + ( 2 * 1024 * 1024 ) )

#define START_GC_EVENT               0
#define END_MARKING_EVENT            1
#define END_COMPACTING_EVENT         2

gcspyMainServer *server;
hsOldDriver *driver;

extern "C" {
static void *
main_loop (void *arg) {
  int counter = 0;

  server->wait_for_client();

  while ( 1 ) {
    gcspyTimer::wait(1000);
    int event = counter % 3;

    if (server->is_connected(event)) {
      server->start_compensation_timer();

      switch (event) {
      case START_GC_EVENT:
	driver->reset(END_ONE);
	driver->set_alloc_limit(LIMIT_ONE);
	break;
      case END_MARKING_EVENT:
	driver->reset(END_TWO);
	driver->set_alloc_limit(LIMIT_TWO);
	break;
      case END_COMPACTING_EVENT:
	driver->reset(END_THREE);
	driver->set_alloc_limit(LIMIT_THREE);
	break;
      }

      server->stop_compensation_timer();

      if (server->is_connected(event)) {
	server->transmit_streams();
      }
    }
    server->safepoint(event);

    ++counter;
  }

  return NULL;
}
}

int
main (int argc, char *argv[]) {
  int res;
#ifdef _SOLARIS_
  thread_t tid;
#endif //_SOLARIS_
#ifdef _LINUX_
  pthread_t tid;
#endif //_LINUX_
  int port;

  if (argc != 2) {
    gcspy_raise_error("Wrong number of arguments");
  }
  port = atoi(argv[1]);

  printf("-- Initialising main server on port %d\n", port);
  
  server = new gcspyMainServer(port, "HS M&C Test Server", MAX_LEN, 1);
  server->add_event(START_GC_EVENT, "Start Full GC");
  server->add_event(END_MARKING_EVENT , "End Marking Phase");
  server->add_event(END_COMPACTING_EVENT ,  "End Compacting Phase");

  driver = new hsOldDriver(BLOCK_SIZE, CARD_SIZE, START, END_ONE);
  server->add_space(driver);

#ifdef _SOLARIS_
  res = thr_create(NULL, 0, main_loop, NULL, THR_BOUND, &tid);
#endif //_SOLARIS_

#ifdef _LINUX_
  res = pthread_create(&tid, NULL, main_loop, NULL);
#endif //_LINUX_

  if (res != 0) {
    printf("Couldn't create thread.\n");
    exit(-1);
  }

  server->main_loop();
}
