/**
 ** Copyright (c) 2002 Sun Microsystems, Inc.
 **
 ** See the file "license.terms" for information on usage and redistribution
 ** of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 **/

#include <stdio.h>

#include "gcspy_buffered_input.hpp"
#include "gcspy_buffered_output.hpp"

int main() {
  char buffer[103];

  char byte_arr[] = { 1, -3, 5, -7, 9 };
  unsigned char ubyte_arr[] = { 0, 50, 100, 150, 200 };
  short short_arr[] = { -1, 2, -3, 4, -5 };
  unsigned short ushort_arr[] = { 111, 222, 333, 444, 555 };
  int int_arr[] = { 100000, -20000, 30000, -40000, 50000 };
  gcspyColor color(100, 3, 54);

  gcspyBufferedOutput output(buffer, 103);

  output.write_byte(-65);
  output.write_ubyte(130);
  output.write_short(-4000);
  output.write_ushort(40000);
  output.write_int(-1000000);
  output.write_string("This is a String!");
  output.write_string("");
  output.write_string(NULL);
  output.write_byte_array(byte_arr, 5);
  output.write_ubyte_array(ubyte_arr, 5);
  output.write_short_array(short_arr, 5);
  output.write_ushort_array(ushort_arr, 5);
  output.write_int_array(int_arr, 5);
  output.write_color(&color);

  output.close();



  printf("buffer length = %u\n", output.get_len());
  printf("\n");

  int i;
  char array_buffer[1024];
  gcspy_length_t len;

  gcspyBufferedInput input(buffer, output.get_len());

  printf("byte = %d\n", input.read_byte());
  printf("ubyte = %d\n", input.read_ubyte());
  printf("short = %d\n", input.read_short());
  printf("ushort = %d\n", input.read_ushort());
  printf("int = %d\n", input.read_int());
  printf("str = [%s]\n", input.read_string(array_buffer));
  printf("str = [%s]\n", input.read_string(array_buffer));
  printf("str = [%s]\n", input.read_string(array_buffer));

  input.read_byte_array((char *) array_buffer, &len);
  printf("byte array, len = %u\n", len);
  for (i = 0; i < len; ++i)
    printf("  [%d] = %d\n", i, ((char *) array_buffer)[i]);

  input.read_ubyte_array((unsigned char *) array_buffer, &len);
  printf("ubyte array, len = %u\n", len);
  for (i = 0; i < len; ++i)
    printf("  [%d] = %d\n", i, ((unsigned char *) array_buffer)[i]);

  input.read_short_array((short *) array_buffer, &len);
  printf("short array, len = %u\n", len);
  for (i = 0; i < len; ++i)
    printf("  [%d] = %d\n", i, ((short *) array_buffer)[i]);

  input.read_ushort_array((unsigned short *) array_buffer, &len);
  printf("ushort array, len = %u\n", len);
  for (i = 0; i < len; ++i)
    printf("  [%d] = %d\n", i, ((unsigned short *) array_buffer)[i]);

  input.read_int_array((int *) array_buffer, &len);
  printf("int array, len = %u\n", len);
  for (i = 0; i < len; ++i)
    printf("  [%d] = %d\n", i, ((int *) array_buffer)[i]);

  input.read_color((gcspyColor *) array_buffer);
  printf("color[%3d, %3d, %3d]\n",
	 ((gcspyColor *) array_buffer)->get_red(),
	 ((gcspyColor *) array_buffer)->get_green(),
	 ((gcspyColor *) array_buffer)->get_blue());

  input.close();
}
