/**
 ** Copyright (c) 2002 Sun Microsystems, Inc.
 **
 ** See the file "license.terms" for information on usage and redistribution
 ** of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 **/

#include <stdio.h>
#include <stdlib.h>

#include "gcspy_buffered_input.hpp"
#include "gcspy_buffered_output.hpp"

void send (gcspyClient *client,
	   char b,
	   unsigned char ub,
	   short s,
	   unsigned short us,
	   int i,
	   const char *str,
	   int mult) {
  gcspyBufferedOutput output = gcspyBufferedOutput(client);
  output.write_byte(b);
  output.write_ubyte(ub);
  output.write_short(s);
  output.write_ushort(us);
  output.write_int(i);
  output.write_string(str);
  for (int n = 0; n < 15; ++n) {
    output.write_int(n * mult);
  }
  output.close();

  gcspy_length_t len = output.get_len();
  printf("== Sending %u bytes", len);
  output.send();
  printf("\n\n");
}

void receive (gcspyClient *client) {
  printf("== Receiving data");
  client->definitely_receive();
  printf("\n");

  gcspy_length_t len = client->get_len();
  printf("==   got %u bytes\n", len);

  gcspyBufferedInput input = gcspyBufferedInput(client);
  char b = input.read_byte();
  unsigned char ub = input.read_ubyte();
  short s = input.read_short();
  unsigned short us = input.read_ushort();
  int i = input.read_int();
  char str[128];
  input.read_string(str);

  printf("==   b = %d, ub = %d, s = %d, us = %d, i = %d, str = %s\n",
	 b, ub, s, us, i, str);
  printf("==   Rest:");
  while (!input.has_finished()) {
    printf(" %d", input.read_int());
  }
  input.close();

  printf("\n\n");
}

int main(int argc, char *argv[]) {
  if (argc != 2) {
    gcspy_raise_error("parameters: <port>");
  }
  int port = atoi(argv[1]);

  gcspyServer server = gcspyServer(port);
  gcspyClient client;

  printf("-- waiting for connection on port %d...\n", port);
  client.wait_for_connection(&server);

  receive(&client);
  receive(&client);
  receive(&client);
  send(&client, (char) 1, (unsigned char) 130, (short) 511,
       (unsigned short) 1076, 2000666, "a C++ string!", 3);
  send(&client, (char) -3, (unsigned char) 5, (short) -600,
       (unsigned short) 100, -1000000, "another C++ string!", -1);
  send(&client, (char) 100, (unsigned char) 255, (short) 20,
       (unsigned short) 50000, 100, "a final C++ string!", 2);

  client.close();
  server.close();
  printf("-- done.");
}
