package gcspy.interpreter;

import gcspy.comm.BufferedInput;
import gcspy.comm.BufferedOutput;

import java.io.IOException;

/**
 * Complete trace information
 **/
public class TraceData {
	
	private TraceHeaderData info;
	private SingleEventTraceData current;

	public TraceData() {
		this(new TraceHeaderData());
	}
	
	public TraceData(TraceHeaderData header) {
		this.info = header;
		this.current = new SingleEventTraceData(this.info.getSpaceNum());
	}
	
	public SingleEventTraceData getCurrent() {
		return this.current;
	}
	
	public Space getSpace(int index) {
		return getCurrent().getSpace(index);
	}
	
	public void setSpace(int index, Space space) {
		getCurrent().setSpace(index, space);
	}
	
	public int getSpaceNum() {
		return this.info.getSpaceNum();
	}

	public void setSpaceNum(int spaceNum) {
		this.info.setSpaceNum(spaceNum);
		getCurrent().resize(spaceNum);
	}

	public Events getEvents() {
		return this.info.getEvents();
	}

	public void setEvents(Events events) {
		this.info.setEvents(events);
	}

	public EventFilters getEventFilters() {
		return this.info.getEventFilters();
	}

	public void setEventFilters(EventFilters eventFilters) {
		this.info.setEventFilters(eventFilters);
	}

	public String getGeneralInfo() {
		return this.info.getGeneralInfo();
	}

	public void setGeneralInfo(String generalInfo) {
		this.info.setGeneralInfo(generalInfo);
	}

	public int[] getEventCount() {
		return getCurrent().getEventCount();
	}

	public void setEventCount(int[] eventCount) {
		getCurrent().setEventCount(eventCount);
	}
	
	/******************** Serialisation / Deserialisation ********************/

	/**
	 * Serialise everything (space number, general information, events, spaces)
	 * 
	 * @param generator
	 *            A generator that will create a new BufferedOutput
	 * @throws IOException
	 */
	public void serialiseEverything(OutputGenerator generator)
			throws IOException {
		BufferedOutput output;

		output = generator.createBufferedOutput();
		info.serialise(output);
		generator.writeBuffer(output.getLen());

		for (int i = 0; i < getSpaceNum(); ++i) {
			Space space = getSpace(i);

			output = generator.createBufferedOutput();
			space.serialise(output);
			generator.writeBuffer(output.getLen());
		}
	}

	/**
	 * Deserialise everything (space number, general information, events,
	 * spaces)
	 * 
	 * @param generator
	 *            A generator that will create a new BufferedInput
	 * @throws IOException
	 */
	public void deserialiseEverything(InputGenerator generator)
			throws IOException {
		
		BufferedInput input;

		input = generator.createBufferedInput();
		info = TraceHeaderData.deserialise(input);
		input.close();
		// although deserialiseHeader sets the space size in the header, it does not resize
		// the actual array. Do this by calling setSpaceNum() with the same number of spaces
		// again.
		setSpaceNum(getSpaceNum());
		for (int i = 0; i < getSpaceNum(); ++i) {
			input = generator.createBufferedInput();
			Space space = Space.deserialise(input);
			setSpace(i, space);
			input.close();
		}
	}

	public int addSpace(Space space) {
		int spaceID = info.getSpaceNum();
		space.setID(spaceID);
		current.addSpace(spaceID, space);
		info.setSpaceNum(spaceID + 1);
		return info.getSpaceNum();
	}

	public TraceHeaderData getHeader() {
		return this.info;
	}	
}
