/**
 ** $Id: FileUtils.java 28 2005-06-20 13:13:35Z rej $
 ** Copyright (c) 2002 Sun Microsystems, Inc.
 **
 ** See the file "license.terms" for information on usage and redistribution
 ** of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 **/

package gcspy.utils;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

/**
 * Utilities for file manipulation
 * 
 * @author Tony Printezis
 */
public class FileUtils {

	private final static int COPY_BUFFER_SIZE = 40 * 1024;
	
	/**
	 * Does a file exist.
	 * 
	 * @param fileName
	 * @return true if it exists
	 */
	static public boolean exists(String fileName) {
		return new File(fileName).exists();
	}
	
	/**
	 * Return the length of a file
	 * 
	 * @param fileName
	 *            The file's path name
	 * @return Its length
	 */
	static public long length(String fileName) {
		File file = new File(fileName);
		return file.length();
	}

	/**
	 * A read from an input stream into a byte array
	 * 
	 * @param in
	 *            The input stream
	 * @param array
	 *            The target array
	 * @param offset
	 *            The offset in the file
	 * @param len
	 *            The number of bytes to read
	 * @throws IOException
	 */
	static public void read(InputStream in, byte array[], int offset, int len)
			throws IOException {
		int soFar = 0;
		while (soFar < len) {
			int curr = in.read(array, offset + soFar, len - soFar);
			soFar += curr;
		}
	}

	public static void seek(FileInputStream f, long numBytesToSkip) throws IOException {
		long toSeek = numBytesToSkip;
		while (toSeek > 0) {
			toSeek -= f.skip(toSeek);
		}
	}

	public static void copy(InputStream in, OutputStream out) throws IOException {
		int count;
		byte[] data = new byte[COPY_BUFFER_SIZE];
		while ((count = in.read(data, 0, data.length)) != -1) {
			out.write(data, 0, count);
		}
	}	
}
