/**
 ** $Id: EventFilterFrame.java 22 2005-06-14 05:17:45Z rej $
 ** Copyright (c) 2002 Sun Microsystems, Inc.
 **
 ** See the file "license.terms" for information on usage and redistribution
 ** of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 **/

package gcspy.vis;

import gcspy.interpreter.EventFilters;
import gcspy.vis.utils.Factory;
import gcspy.vis.utils.OKCancelDialog;

import java.awt.BorderLayout;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JSpinner;
import javax.swing.JTable;
import javax.swing.JToggleButton;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;

import net.miginfocom.swing.MigLayout;

/**
 * Dialog for configuring event filters.
 * 
 * @author Tony Printezis
 * @author Peter Hofer
 */
class EventFilterDialog extends OKCancelDialog {
	private EventFilters eventFilters;

	private JTable table;
	private EventFilterTableModel model;

	private JPopupMenu resetM;
	private JMenuItem defaultsMI;
	private JMenuItem enableAllMI;
	private JMenuItem disableAllMI;
	private JMenuItem clearDelaysMI;
	private JMenuItem clearPausesMI;
	private JMenuItem resetPeriodsMI;

	private JToggleButton resetB;

	private ActionListener menuItemListener = new ActionListener() {
		@Override
		public void actionPerformed(ActionEvent e) {
			Object source = e.getSource();
			if (source == defaultsMI) {
				model.revertToDefaults();
			} else if (source == enableAllMI) {
				model.enableAll();
			} else if (source == disableAllMI) {
				model.disableAll();
			} else if (source == clearDelaysMI) {
				model.clearDelays();
			} else if (source == clearPausesMI) {
				model.clearPauses();
			} else if (source == resetPeriodsMI) {
				model.resetPeriods();
			}
		}
	};

	private ActionListener resetBListener = new ActionListener() {
		public void actionPerformed(ActionEvent event) {
			// Show menu
			resetM.show(resetB, 0, resetB.getHeight());
		}
	};

	private PopupMenuListener menuListener = new PopupMenuListener() {
		@Override
		public void popupMenuCanceled(PopupMenuEvent e) {
		}

		@Override
		public void popupMenuWillBecomeInvisible(PopupMenuEvent e) {
			// Put menu button in untoggled mode
			resetB.setSelected(false);
		}

		@Override
		public void popupMenuWillBecomeVisible(PopupMenuEvent e) {
			// Put menu button in toggled mode
			resetB.setSelected(true);
		}
	};

	/**
	 * Returns the current event filter configuration including all
	 * modifications of the user.
	 */
	public EventFilters getEventFilters() {
		return eventFilters;
	}

	private void setup(String eventNames[]) {
		setTitle("Event Filters");
		setResizable(false);
		setLayout(new BorderLayout());

		model = new EventFilterTableModel(eventNames, eventFilters);
		table = Factory.createTable(model);
		table.getColumnModel().getColumn(0).setPreferredWidth(200);
		table.getColumnModel().getColumn(2).setCellRenderer(new SpinnerTableCellRenderer());
		table.getColumnModel().getColumn(2).setCellEditor(
			new SpinnerTableCellEditor(new JSpinner(new SpinnerNumberModel(EventFilters.DELAY_DEFAULT,
					EventFilters.DELAY_MIN, EventFilters.DELAY_MAX, 10))));
		table.getColumnModel().getColumn(4).setCellRenderer(new SpinnerTableCellRenderer());
		table.getColumnModel().getColumn(4).setCellEditor(
			new SpinnerTableCellEditor(new JSpinner(new SpinnerNumberModel(EventFilters.PERIOD_DEFAULT,
					EventFilters.PERIOD_MIN, EventFilters.PERIOD_MAX, 1))));
		table.putClientProperty("terminateEditOnFocusLost", Boolean.TRUE);

		add(table, BorderLayout.CENTER);
		add(table.getTableHeader(), BorderLayout.NORTH);

		// Menu
		resetM = new JPopupMenu("Reset");
		resetM.addPopupMenuListener(menuListener);

		defaultsMI = new JMenuItem("Restore Defaults");
		defaultsMI.addActionListener(menuItemListener);
		resetM.add(defaultsMI);

		enableAllMI = new JMenuItem("Enable All Events");
		enableAllMI.addActionListener(menuItemListener);
		resetM.add(enableAllMI);

		disableAllMI = new JMenuItem("Disable All Events");
		disableAllMI.addActionListener(menuItemListener);
		resetM.add(disableAllMI);

		clearDelaysMI = new JMenuItem("Reset Delays");
		clearDelaysMI.addActionListener(menuItemListener);
		resetM.add(clearDelaysMI);

		clearPausesMI = new JMenuItem("Clear Pauses");
		clearPausesMI.addActionListener(menuItemListener);
		resetM.add(clearPausesMI);

		resetPeriodsMI = new JMenuItem("Reset Periods");
		resetPeriodsMI.addActionListener(menuItemListener);
		resetM.add(resetPeriodsMI);

		// Buttons
		JPanel buttonPanel = getButtonPanel();
		resetB = new JToggleButton("Reset");
		resetB.addActionListener(resetBListener);
		buttonPanel.add(resetB);

		JPanel buttons = new JPanel(new MigLayout("fill"));
		buttons.add(buttonPanel, "grow");
		add(buttons, BorderLayout.SOUTH);

		pack();
	}

	/**
	 * Create a new event filter frame. The specified event filter configuration
	 * will not be modified, instead the changed configuration can be obtained
	 * with {@link #getEventFilters()}.
	 * 
	 * @param owner
	 *            The parent frame
	 * @param eventNames
	 *            The event names
	 * @param eventFilters
	 *            The current event filter configuration.
	 */
	EventFilterDialog(Window owner, String eventNames[], EventFilters eventFilters) {
		super(owner);

		this.eventFilters = (EventFilters) eventFilters.clone();
		setup(eventNames);
	}

}
