package gcspy.vis;

import java.awt.Component;
import java.awt.event.MouseEvent;
import java.text.ParseException;
import java.util.EventObject;

import javax.swing.AbstractCellEditor;
import javax.swing.JSpinner;
import javax.swing.JTable;
import javax.swing.table.TableCellEditor;

/**
 * Cell editor that uses a {@link JSpinner} for editing values.
 * 
 * @author Peter Hofer
 */
public class SpinnerTableCellEditor extends AbstractCellEditor implements TableCellEditor {

	private final JSpinner spinner;
	private Object oldValue;

	/**
	 * Constructs a {@link SpinnerTableCellEditor} using a {@link JSpinner}
	 * instance initialized with its default constructor.
	 * {@link JSpinner#JSpinner()}
	 */
	public SpinnerTableCellEditor() {
		this(new JSpinner());
	}

	/**
	 * Constructs a {@link SpinnerTableCellEditor} using the specified
	 * {@link JSpinner} instance.
	 * 
	 * @param spinner
	 *            The spinner to use.
	 */
	public SpinnerTableCellEditor(JSpinner spinner) {
		if (spinner == null)
			throw new NullPointerException("spinner");

		this.spinner = spinner;
	}

	@Override
	public void cancelCellEditing() {
		spinner.setValue(oldValue);
		oldValue = null;
		super.cancelCellEditing();
	}

	@Override
	public boolean stopCellEditing() {
		boolean committed = true;
		try {
			spinner.commitEdit();
		} catch (ParseException e) {
			// invalid value
			committed = false;
		}

		if (committed) {
			oldValue = null; // clear reference
			fireEditingStopped();
		}
		return committed;
	}

	@Override
	public Object getCellEditorValue() {
		return spinner.getValue();
	}

	@Override
	public boolean isCellEditable(EventObject e) {
		if (e instanceof MouseEvent) {
			return (((MouseEvent) e).getClickCount() > 0);
		}
		return false;
	}

	@Override
	public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) {
		spinner.setValue(value);
		oldValue = value;
		return spinner;
	}
}
