package gcspy.vis;

import gcspy.interpreter.Space;
import gcspy.interpreter.Stream;
import gcspy.vis.TileComponent.TileFlags;
import gcspy.vis.TileComponent.TileType;
import gcspy.vis.utils.Factory;

import java.awt.Font;
import java.util.ArrayList;
import java.util.EnumSet;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;

import net.miginfocom.swing.MigLayout;

/**
 * Component that displays a legend for the values of a stream.
 * 
 * @author Peter Hofer
 */
public class StreamLegend extends JPanel {

	/**
	 * Creates a new {@link StreamLegend} object.
	 * 
	 * @param space
	 *            The space of the stream to display.
	 * @param streamId
	 *            Index of the stream in the space.
	 * @param style
	 *            Style in which to render the legend.
	 */
	public StreamLegend(Space space, int streamId, TileStyle style) {
		Stream stream = space.getStream(streamId);
		int min = stream.getMinValue();
		int max = stream.getMaxValue();
		int presentation = stream.getPresentation();

		setBorder(BorderFactory.createTitledBorder(stream.getName()));

		// Determine the values to display
		List<Integer> values = new ArrayList<Integer>();
		switch (presentation) {
		case Stream.PRESENTATION_PLUS:
			values.add(max + 1);
			// intended fall-through
		case Stream.PRESENTATION_PLAIN:
		case Stream.PRESENTATION_MAX_VAR:
		case Stream.PRESENTATION_PERCENT:
			values.add(max);
			values.add((min + max) / 2); // half
			values.add(min);
			break;
		case Stream.PRESENTATION_PERCENT_VAR:
			values.add(100);
			values.add(50);
			values.add(0);
			break;
		case Stream.PRESENTATION_ENUM:
			for (int i = max; i >= 0; --i)
				values.add(i);
			break;
		}

		setLayout(new MigLayout());
		for (int value : values) {
			int adjusted = value;
			if (presentation != Stream.PRESENTATION_PERCENT_VAR) {
				adjusted = stream.adjustValue(value);
			}

			TileComponent tile = new TileComponent(style);
			if (stream.isOverflow(value)) {
				tile.setTile(space, streamId, TileType.OVERFLOW,
						EnumSet.noneOf(TileFlags.class), adjusted);
			} else {
				tile.setTile(space, streamId, TileType.VALUE,
						EnumSet.noneOf(TileFlags.class), adjusted);
			}
			add(tile, "wmax pref, hmax pref");

			JLabel label = Factory.createLabel(stream.presentDataSmall(value), Font.PLAIN,
					(presentation == Stream.PRESENTATION_ENUM) ? JLabel.LEFT : JLabel.RIGHT);
			add(label, "push, wrap");
		}
	}
}
