package gcspy.vis.utils;

import java.io.OutputStream;
import java.io.PrintWriter;

import javax.swing.JTable;
import javax.swing.table.TableModel;

/**
 * A simple exporter for {@link TableModel} instances (and therefore,
 * {@link JTable} components).
 * 
 * @author Peter Hofer
 * 
 */
public class TableExporter {

	/**
	 * Column separator for CSV files.
	 */
	public static final String CSV_COLUMN_SEPARATOR = ",";

	/**
	 * Platform-dependent row separator.
	 */
	public static final String CSV_ROW_SEPARATOR = System.getProperty("line.separator");

	/**
	 * Extract values from the specified table model and write them to the given
	 * output stream.
	 * 
	 * @see #exportTable(OutputStream, TableModel, boolean, String, String)
	 * @see #CSV_ROW_SEPARATOR
	 * @see #CSV_COLUMN_SEPARATOR
	 */
	public static void exportTableCSV(OutputStream stream, TableModel model, boolean headers) {
		exportTable(stream, model, headers, CSV_COLUMN_SEPARATOR, CSV_ROW_SEPARATOR);
	}

	/**
	 * Write values for all cells obtained from the table model to the specified
	 * stream, with colSep separating columns and rowSep separating rows.
	 * 
	 * @param stream
	 *            The stream to write the text data to.
	 * @param model
	 *            The model to extract the data from.
	 * @param headers
	 *            Whether to include the table header names as the first row.
	 * @param colSep
	 *            Column separator.
	 * @param rowSep
	 *            Row separator.
	 */
	public static void exportTable(OutputStream stream, TableModel model, boolean headers, String colSep, String rowSep) {
		PrintWriter writer = new PrintWriter(stream);

		int nrows = model.getRowCount();
		int ncols = model.getColumnCount();
		if (headers) {
			for (int i = 0; i < ncols; ++i) {
				writer.append(model.getColumnName(i));
				writer.append(colSep);
			}
			writer.append(rowSep);
		}
		for (int i = 0; i < nrows; ++i) {
			for (int j = 0; j < ncols; ++j) {
				Object value = model.getValueAt(i, j);
				if (value == null)
					value = "(null)";

				writer.append(value.toString());
				writer.append(colSep);
			}
			writer.append(rowSep);
		}
	}

}
